/*
   Copyright (C) 2001-2003 Thomas Capricelli <orzel@freehackers.org>
*/

#include <opale_factory.h>
#include <opale_part.h>
#include <kaboutdata.h>
#include <kinstance.h>
#include <kiconloader.h>
#include <klocale.h>
#include <kdebug.h>

extern "C"
{
    void* init_libopalepart()
    {
        return new OpaleFactory;
    }
};

static const char* description=I18N_NOOP("Very simple personal finance manager");
static const char* version="0.5";

KInstance* OpaleFactory::s_global = 0L;
KAboutData* OpaleFactory::s_aboutData = 0L;

OpaleFactory::OpaleFactory( QObject* parent, const char* name )
    : KoFactory( parent, name )
{
    global();
}

OpaleFactory::~OpaleFactory()
{
    if ( s_aboutData )
    {
      delete s_aboutData;
      s_aboutData = 0L;
    }
    if ( s_global )
    {
      delete s_global;
      s_global = 0L;
    }
}

KParts::Part* OpaleFactory::createPartObject( QWidget *parentWidget, const char *widgetName, QObject* parent, const char* name, const char* classname, const QStringList & )
{
    // If classname is "KoDocument", our host is a koffice application
    // otherwise, the host wants us as a simple part, so switch to readonly and single view.
    bool bWantKoDocument = ( strcmp( classname, "KoDocument" ) == 0 );

    // parentWidget and widgetName are used by KoDocument for the "readonly+singleView" case.
    OpalePart *part = new OpalePart( parentWidget, widgetName, parent, name, !bWantKoDocument );

    if ( !bWantKoDocument )
      part->setReadWrite( false );

    return part;
}

KAboutData* OpaleFactory::aboutData()
{
    if ( !s_aboutData )
    {
    	// Change this, of course
    	s_aboutData = new KAboutData( "opale", "Opale",
            version, description, KAboutData::License_GPL,
            "(c) 2001-2003, Thomas Capricelli", 0, "http://orzel.freehackers.org/opale");

    	    s_aboutData->addAuthor("Thomas Capricelli",0,
	    "orzel@freehackers.org", "http://orzel.freehackers.org");
    }
    return s_aboutData;
}

KInstance* OpaleFactory::global()
{
    if ( !s_global )
    {
        s_global = new KInstance( aboutData() );
        // Add any application-specific resource directories here

        // Tell the iconloader about share/apps/koffice/icons
        s_global->iconLoader()->addAppDir("koffice");
    }
    return s_global;
}

