/*
   Copyright (C) 2001-2003 Thomas Capricelli <orzel@freehackers.org>
*/

/* Qt */
#include <qvaluelist.h>
#include <qpainter.h>
#include <qvbox.h>
#include <qpushbutton.h>

/* KDE */
#include <kdebug.h>
#include <klocale.h>

/* KOffice */
#include <KDChartData.h>
#include <KDChartTable.h>


/* Opale */
#include "opale_graph.h"
#include "opale_part.h"

#define OPALE_GRAPH_MIN_LEN (60*60*24*5) 	// 5 days

OpaleGraph::OpaleGraph( OpalePart *_part, QWidget *parentWidget, const char *widgetName)
	: KMainWindow( parentWidget, widgetName)
	, part(_part)
	, chartData(5, 10 )
{
/*
	if (lp->count() < 5)
		kdDebug(OPALE_DEBUG_STREAM) <<  "strange, lp->count is small : " << lp->count() << endl;
*/

	// default start/end = +/- 2 months
	start = QDateTime::currentDateTime().addMonths(-2);
	end   = QDateTime::currentDateTime().addMonths(+2);

	// configure the chart
	chartParams.setChartType(KDChartParams::Line);
	chartParams.setLineChartSubType(KDChartParams::LineNormal);
	chartParams.setPrintDataValues(false);
	chartParams.setLegendTitleText( i18n("Legend") ); // this one is not i18n'ed in kdchart..
	chartParams.setLegendText(0, i18n("Balance") );
	chartParams.setLegendText(1, i18n("Best case") );
	chartParams.setLegendText(2, i18n("Worst case") );
	chartParams.setLegendText(3, i18n("Best case (from now on)") );
	chartParams.setLegendText(4, i18n("Worst case (from now on)") );
	chartParams.setLineWidth(2);
	chartParams.setLegendPosition(KDChartParams::LegendTop);

	// layout
	QHBox * hb = new QHBox(this);
	QVBox * vb = new QVBox(hb);

	// do it
	chart = new KDChartWidget(&chartParams, &chartData, hb, "opale_chart");
	hb->setStretchFactor(vb,1);
	hb->setStretchFactor(chart,8);

//	kdDebug(OPALE_DEBUG_STREAM) <<  "Trying to find reason for ba:ck bgd"<< endl;
//	kdDebug(OPALE_DEBUG_STREAM) << "brush : " << chart->backgroundBrush() << endl;
//	kdDebug(OPALE_DEBUG_STREAM) << "color : " << chart->backgroundColor() << endl;
	chart->setBackgroundMode( Qt::FixedColor  );
	chart->setPaletteBackgroundColor( Qt::white  );

	connect( new QPushButton( i18n("Zoom in"), vb), SIGNAL(clicked(void)), this ,SLOT(zoomin(void)));
	connect( new QPushButton( i18n("Zoom out"), vb), SIGNAL(clicked(void)), this ,SLOT(zoomout(void)));
	connect( new QPushButton( i18n("Go left"), vb), SIGNAL(clicked(void)), this ,SLOT(goleft(void)));
	connect( new QPushButton( i18n("Go right"), vb), SIGNAL(clicked(void)), this ,SLOT(goright(void)));

	// polishing
	setCentralWidget(hb);
	setBeginEnd( start.toTime_t(), end.toTime_t() ); // update middle/len
	setMinimumSize(500,400);
}

OpaleGraph::~OpaleGraph()
{
	delete chart;
}

void OpaleGraph::updateChart(void)
{
	amount_t a_min, a_max;
	part->fillDataGraph(chartData, start, end, a_min, a_max);

/*
       kdDebug(OPALE_DEBUG_STREAM) <<  " in updateChart() before : valuestart/stop are "
              << chartParams.axisParams(KDChartAxisParams::AxisPosBottom).axisValueStart().dateTimeValue(2)
              << " and "
              << chartParams.axisParams(KDChartAxisParams::AxisPosBottom).axisValueEnd().dateTimeValue(2)
              << endl;
*/

	// configure X axis position
	KDChartAxisParams axis(chartParams.axisParams(KDChartAxisParams::AxisPosBottom));
	axis.setAxisValueStart( KDChartData(0,start));
	axis.setAxisValueEnd( KDChartData(0,end));
	chartParams.setAxisParams(KDChartAxisParams::AxisPosBottom, axis);

/*
       kdDebug(OPALE_DEBUG_STREAM) <<  " in updateChart() after: valuestart/stop are "
              << chartParams.axisParams(KDChartAxisParams::AxisPosBottom).axisValueStart().dateTimeValue(2)
              << " and "
              << chartParams.axisParams(KDChartAxisParams::AxisPosBottom).axisValueEnd().dateTimeValue(2)
              << endl;
*/

	// i can't figure this KDChart configuration stuff, so i compute ymin/max myself..
	// configure Y axis position
	axis = chartParams.axisParams(KDChartAxisParams::AxisPosLeft);
	axis.setAxisValueStart( KDChartData(a_min/100.));
	axis.setAxisValueEnd( KDChartData(a_max/100.));
	chartParams.setAxisParams(KDChartAxisParams::AxisPosLeft, axis);

	// it seems a update() wont' be enough here :
	chart->repaint();
}


void OpaleGraph::mousePressEvent( QMouseEvent *evt )
{
	kdDebug(OPALE_DEBUG_STREAM) <<  " mousePressEvent() x is " << evt->x() << endl;
}


void OpaleGraph::mouseReleaseEvent( QMouseEvent *evt )
{
	kdDebug(OPALE_DEBUG_STREAM) <<  " mouseReleaseEvent() x is " << evt->x() << endl;

}

/*
	if ( x_pressed == evt->x() - draw_offset ) {
		//kdDebug(OPALE_DEBUG_STREAM) <<  "OpaleGraph::mouseReleaseEvent, simple zooming" << evt->x() << endl;
		int center = xinverse_transform( x_pressed );
		int halfwidth = time_max - time_min;
	
		// left zoom in *2, right zoom out /2
		if ( Qt::LeftButton & evt->button())
			halfwidth /= 4;
		
		//kdDebug(OPALE_DEBUG_STREAM) <<  " center, halfwidth :  " << center << ", " << halfwidth  << endl;
		normalize( center-halfwidth, center+halfwidth);
	} else {
		//kdDebug(OPALE_DEBUG_STREAM) <<  "OpaleGraph::mouseReleaseEvent, drag&drop zooming" << evt->x() << endl;

		if ( ! (Qt::LeftButton & evt->button()) )
			return;

		int x1 = xinverse_transform( x_pressed );
		int x2 = xinverse_transform( evt->x() - draw_offset );
		if (x1>x2) { // x1 <-> x2, whithout temporary
			x1 -= x2;
			x2 += x1;
			x1 = x2 - x1;
			}
		// drag&drop zooming
		normalize( x1, x2 );
	}
	render();
}

*/

#if 0
	// might be needed 

void OpaleGraph::mouseMoveEvent( QMouseEvent *evt )
{
//	kdDebug(OPALE_DEBUG_STREAM) <<  " mouseMoveEvent() x is " << evt->x() << endl;
}
#endif

void OpaleGraph::goleft(void)
{
	setBeginEnd(middle-len, middle);
}

void OpaleGraph::goright(void)
{
	setBeginEnd(middle, middle+len);
}

void OpaleGraph::zoomin(void)
{
	uint _len = len/2;

	if ( _len < OPALE_GRAPH_MIN_LEN)
		_len = OPALE_GRAPH_MIN_LEN;

	setBeginEnd(middle-_len/2, middle+_len/2);
}

void OpaleGraph::zoomout(void)
{
	setBeginEnd(middle-len, middle+len);
}

void OpaleGraph::setBeginEnd(uint uistart, uint uiend)
{
//	kdDebug(OPALE_DEBUG_STREAM) <<  " moving from "
//		<< start.toString() << " " << end.toString() << endl;

	start.setTime_t(uistart);
	end.setTime_t(uiend);

	// updating len/middle
	len = uiend - uistart;
	middle = (uiend+uistart)/2;

//	kdDebug(OPALE_DEBUG_STREAM) << " to " << start.toString() << " " << end.toString() << endl;


	// updating chart
	updateChart();
}


