/*
   Copyright (C) 2001-2003 Thomas Capricelli <orzel@freehackers.org>
*/

#ifndef OPALE_PART_H
#define OPALE_PART_H

/* Qt */
#include <qlist.h>
#include <qptrdict.h>

/* KDE */

/* KOffice */
#include <KDChartTable.h>
#include <koDocument.h>

/* Opale */
#include "opale_record.h"


class QComboBox;
class KListView;
class QDateTime;

class OpalePart : public KoDocument
{
    Q_OBJECT
public:
    OpalePart( QWidget *parentWidget = 0, const char *widgetName = 0, QObject* parent = 0, const char* name = 0, bool singleViewMode = false );

    virtual void paintContent( QPainter &painter, const QRect &rect, bool transparent = false, double zoomX = 1.0, double zoomY = 1.0 );


    /* document handling */
    virtual bool initDoc();
    virtual bool loadXML( QIODevice *, const QDomDocument & );
    virtual QDomDocument saveXML();

    QPtrDict<OpaleRecord> *fillList(Total *, KListView *);
    void	fillDataGraph(KDChartTableData &chartData , QDateTime &begin, QDateTime &end, amount_t &a_min, amount_t &a_max);
    QString	currency(void) {return s_currency; }
    QString	account_name(void) {return s_account_name; }
    void	setInfos(QString currency, QString account_name) { s_currency = currency; s_account_name = account_name; setModified(true); }
    void		fillCombos(QComboBox *, QComboBox *);
    int			checkTypeCombos(QComboBox *);
    int			checkCategoryCombos(QComboBox *);

    void	deleteRecord(OpaleRecord *);
    void	addRecord(OpaleRecord *);
    void	addPattern(OpalePattern *);
    /** something has changed : */
    void	touched(void);
    OpaleRecord	*getRecordFromPattern(int i) { return new OpaleRecord(*patterns.at(i)); }
    int		editFileOptions(QWidget *parent= 0);
    int         recordsCount( void ) { return records.count() ; }

signals:
    void	newPatterns (QStringList );
    void	dataUpdated(void);

protected:
    virtual KoView* createViewInstance( QWidget* parent, const char* name );
    
    void	syncPatterns(void);

    QPtrList<OpaleRecord>	records;
    QPtrList<OpalePattern>	patterns;
    QStringList			sl_howlist;
    QStringList			sl_categorylist;

    /* account information */
    QString			s_currency;
    QString			s_account_name;
};

#endif
