/*
   Copyright (C) 2001-2003 Thomas Capricelli <orzel@freehackers.org>
*/

/* libc */
#include <math.h>	// HUGE_VAL

/* qt */
#include <qpainter.h>
#include <qlineedit.h>
#include <qvalidator.h>
#include <qcombobox.h>

/* kde */
#include <klistview.h>
#include <kdatepik.h>
#include <koDocument.h>
#include <kdebug.h>

/* opale */
#include "opale_record.h"
#include "input_record.h"
#include "input_pattern.h"



/*
 * OpalePattern
 */


OpalePattern::OpalePattern(QString _name, amount_t _amount)
{
	b_valid = false;

	name = _name;
	m_amount = _amount;
	type = 0;
	category = 0;
}


OpalePattern::OpalePattern(OpalePattern &other)
{
	b_valid = false;

	name = other.name;
	m_amount = other.m_amount;
	type = other.type;
	category = other.category;
}


OpalePattern::OpalePattern(const QDomElement &e)
{
	bool ok;
	b_valid = true;

	if ( e.isNull() || (e.tagName() != "record" && e.tagName() != "pattern") ) {  
		b_valid=false;
		return;
	}

	m_amount = e.attribute("amount").toInt( &ok );
	if (!ok) {
		m_amount = 0;
		b_valid = false;
	}
	name = e.attribute("what");
	type = e.attribute("how").toInt( &ok );
	category = e.attribute("category").toInt( &ok );
	if (!ok) {
		type = -1;
		b_valid = false;
	}
}

#ifndef USE_BUGGY_STRTOD
static amount_t atoprice (const char *buf); // from original cash
#endif

OpalePattern::OpalePattern(inputPatternDialog *dialog, int _type, int _cat)
{
	b_valid = true;

	name = dialog->edit_what->text();
//	type = dialog->edit_how->currentItem();
	type = _type;
	category = _cat;

#ifdef USE_BUGGY_STRTOD
	extern "C" { extern int printf(...); }

	double	f;
	bool	ok;

	f =  dialog->edit_amount->text().toDouble(&ok);
	printf("XXX 0 XXX bool is %s\n", ok?"true":"false");
	printf("XXX 1 XXX %s\n", dialog->edit_amount->text().latin1());
	printf("XXX 1 XXX %f\n", f);

	for (int i = 0; i<sizeof(double); i++) printf("f[%d]=%d\n", i, (unsigned int)((char *)(&f))[i] );
//	f=-f;printf("\n"); 
//	for (int i = 0; i<sizeof(double); i++) printf("f[%d]=%d\n", i, (unsigned int)((char *)(&f))[i] );

	f*=100.; printf("\n");
	for (int i = 0; i<sizeof(double); i++) printf("f[%d]=%d\n", i, (unsigned int)((char *)(&f))[i] );

	printf("XXX 2 XXX %f\n", f);
	printf("XXX 3 XXX %d\n", (int)f);
	m_amount = (int)f;
	printf("amount : %d\n", m_amount);
#else
	m_amount = atoprice(dialog->edit_amount->text().latin1());
#endif

}


QDomElement OpalePattern::domElement(QDomDocument doc)
{
	QDomElement record = doc.createElement( "pattern" );

	record.setAttribute("amount", (int)m_amount);
	record.setAttribute("what", name);
	record.setAttribute("how", type);
	record.setAttribute("category", category);

	return record;
}


void OpalePattern::to_dialog(inputPatternDialog *dialog)
{
	dialog->edit_what->setText(name);
	dialog->edit_amount->setText(QString().sprintf("%.2f", m_amount/100.));
	dialog->edit_amount->setValidator(new QDoubleValidator(-HUGE_VAL,HUGE_VAL,2, dialog->edit_amount));
	dialog->edit_how->setCurrentItem(type);
	dialog->edit_category->setCurrentItem(category);
}



/*
 * OpaleRecord
 */

OpaleRecord::OpaleRecord(QString _name, amount_t _amount)
	: OpalePattern(_name, _amount)
{
	qdt = QDateTime::currentDateTime();
	_checked = false;
}

OpaleRecord::OpaleRecord (OpalePattern &p)
       	: OpalePattern(p) 
{
	qdt = QDateTime::currentDateTime();
	_checked = false;
}

OpaleRecord::OpaleRecord (OpaleRecord &r)
       	: OpalePattern(r) 
{
	qdt = r.qdt;
	_checked = r.checked();
}


OpaleRecord::OpaleRecord(const QDomElement &e)
	:OpalePattern(e)
{
	bool ok;

	if ( e.isNull() || e.tagName() != "record" ) {  
		b_valid=false;
		return;
	}

	qdt.setTime_t(  e.attribute("when").toInt( &ok ) );
	if (!ok) {
		b_valid = false;
	}
	_checked = e.attribute("checked") == QString("yes");
}


OpaleRecord::OpaleRecord(inputRecordDialog *dialog, int _type, int _cat)
{
	QDateTime	_origin;
	_origin.setTime_t(0l);

	b_valid = true;

	name = dialog->edit_what->text();
//	type = dialog->edit_how->currentItem();
	type = _type;
	category = _cat;

#ifdef USE_BUGGY_STRTOD
	extern "C" { extern int printf(...); }

	double	f;
	bool	ok;

	f =  dialog->edit_amount->text().toDouble(&ok);
	printf("XXX 0 XXX bool is %s\n", ok?"true":"false");
	printf("XXX 1 XXX %s\n", dialog->edit_amount->text().latin1());
	printf("XXX 1 XXX %f\n", f);

	for (int i = 0; i<sizeof(double); i++) printf("f[%d]=%d\n", i, (unsigned int)((char *)(&f))[i] );
//	f=-f;printf("\n"); 
//	for (int i = 0; i<sizeof(double); i++) printf("f[%d]=%d\n", i, (unsigned int)((char *)(&f))[i] );

	f*=100.; printf("\n");
	for (int i = 0; i<sizeof(double); i++) printf("f[%d]=%d\n", i, (unsigned int)((char *)(&f))[i] );

	printf("XXX 2 XXX %f\n", f);
	printf("XXX 3 XXX %d\n", (int)f);
	m_amount = (amount_t)f;
	printf("amount : %d\n", m_amount);
#else
	m_amount = atoprice(dialog->edit_amount->text().latin1());
#endif

	_checked = false;
	qdt = dialog->edit_date->date();

}


QDomElement OpaleRecord::domElement(QDomDocument doc)
{
	QDomElement record = doc.createElement( "record" );

	record.setAttribute("amount", (int)m_amount);
	record.setAttribute("when",  qdt.toTime_t());
	record.setAttribute("what", name);
	record.setAttribute("how", type);
	record.setAttribute("category", category);
	record.setAttribute("checked", checked()?"yes":"no");

	return record;
}



void OpaleRecord::to_dialog(inputRecordDialog *dialog)
{
	dialog->edit_what->setText(name);
	dialog->edit_amount->setText(QString().sprintf("%.2f", m_amount/100.));
	dialog->edit_amount->setValidator(new QDoubleValidator(-HUGE_VAL,HUGE_VAL,2, dialog->edit_amount));
	dialog->edit_how->setCurrentItem(type);
	dialog->edit_category->setCurrentItem(category);

	dialog->edit_date->setDate(qdt.date());
}


KListViewItem * OpaleRecord::fillListView(Total *t, KListView *qlv, QString currency, QStringList &howlist, QStringList &categorylist)
{
	QDate		date = qdt.date();
	KListViewItem	*_item = NULL;

	t->global	+= m_amount;

	_item = new KListViewItem( qlv,
		QString().sprintf("%02d/%02d/%04d", date.day(), date.month(), date.year() ), 
		name,
		QString("%1 %2").arg( QString().sprintf("%12.2f", m_amount/100.)).arg(currency),
		checked()?"X":"",
		howlist[type],
		isFutur()?i18n("Future"):"",
		QString("%1 %2").arg( QString().sprintf("%12.2f", t->global/100.)).arg(currency),
		categorylist[category] );

	if (isPast()) {
		t->utt		+= m_amount;
		t->uttsafe	+= safeAmount();
	}
	if (_checked)
		t->checked	+= m_amount;

	return _item;
}

bool OpaleRecord::isFutur(void)
{
	return ( QDateTime::currentDateTime() < qdt );
}

/**
  *  add some time to the current record
  *  @param unit is 0 for day, 1 for week, 2 for month, and 3 for year
  *  @param nb is how many of those needs to be added
  */
void OpaleRecord::addTime(int nb, int unit)
{
	switch(unit) {
		default:
			kdDebug(OPALE_DEBUG_STREAM) << "unknown unit in OpaleRecord::addTime" << endl;
			printf("DEBUG : BEURK DEFAULT\n");
			return;
		case 0:
			qdt=qdt.addDays(nb);
			break;
		case 1:
			qdt=qdt.addDays(7*nb);
			break;
		case 2:
			qdt=qdt.addMonths(nb);
			break;
		case 3:
			qdt=qdt.addYears(nb);
			break;
	}
}



/*
 * buggy stuff
 */

#ifndef USE_BUGGY_STRTOD

#include <stdlib.h>
#include <ctype.h>
#include <math.h>
#include <values.h> 	// MAXLONG

amount_t
atoprice (const char *buf)
{
  char *w;
  long  a, b;
 
  w = strchr (buf, '.');
 
  if (w)
    {
      *(w + 3) = '\0';
      b = atol (w + 1);
      if (!isdigit (*(w + 2)))
        b *= 10;
      *w = '\0';
      a = atol (buf);
 
      /* provisoire */
      if (a > MAXLONG / 100)
	kdDebug(OPALE_DEBUG_STREAM) << "Price out of limits(1)" << endl;
 
      return (a >= 0) ? 100 * a + b : 100 * a - b;
    }
  else
    {
      /* provisoire */
      a = atol (buf);
      if (a > ((MAXLONG) / 100))
        kdDebug(OPALE_DEBUG_STREAM) << "Price out of limits(2)" << endl;
      return (100 * a);
    }
}
#endif

