/*
   Copyright (C) 2001-2003 Thomas Capricelli <orzel@freehackers.org>
*/

#ifndef OPALE_RECORD
#define OPALE_RECORD

/* unix */
#include <time.h>

/* qt */
#include <qstring.h>
#include <qdatetime.h>

/* KDE */
#include <klocale.h>

/* opale */
#include "opale.h"

class QPainter;
class QDomElement;
class QDomDocument;
class QStringList;
class OpalePart;
class KListView;
class KListViewItem;
class inputRecordDialog;
class inputPatternDialog;


class OpalePattern
{
	friend class OpalePart;

public:
	OpalePattern(QString _name="<empty>", amount_t _amount=0);
	OpalePattern(const QDomElement &);
	OpalePattern(inputPatternDialog *, int how, int cat);
	OpalePattern(OpalePattern &other);

	bool		isValid(void) { return b_valid; }
	amount_t	amount(void) { return m_amount; }

	QDomElement	domElement(QDomDocument);
	void		to_dialog(inputPatternDialog *);

protected:
	bool		b_valid;
	QString		name;
	amount_t	m_amount;
	int		type;
	int		category;
};


class OpaleRecord : public OpalePattern
{
	friend class OpalePart;
public:
	OpaleRecord(QString _name="<empty>", amount_t _amount=0);
	OpaleRecord (const QDomElement &);
	OpaleRecord (inputRecordDialog *, int, int);
	OpaleRecord (OpalePattern &p);
	OpaleRecord (OpaleRecord &r);

	QDomElement	domElement(QDomDocument);
	void		to_dialog(inputRecordDialog *);

	KListViewItem *	fillListView(Total *t, KListView *, QString currency, QStringList &, QStringList &);

	void	check(void) {_checked = true;}
	void	unCheck(void) {_checked = false;}
	bool	checked(void) { return _checked; }
	
	amount_t	safeAmount(void) { return (_checked||m_amount<0)?m_amount:0l; }
	bool		isFutur(void);
	bool		isPast(void) { return !isFutur(); }
	/**
	  *  add some time to the current record
	  *  @param unit is 0 for day, 1 for week, 2 for month, and 3 for year
	  *  @param nb is how many of those needs to be added
	  */
	void		addTime(int nb, int unit);

private:
	QDateTime	qdt;
	bool		_checked;
};

#endif /* OPALE_RECORD */
